dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_generic_functions.lua")
dofile_once("mods/TheArmory/files/trinkets_list.lua")

local entity_id = GetUpdatedEntityID()
entity_id = EntityGetRootEntity( entity_id )
local x, y = EntityGetTransform(entity_id)
local field_id = EntityLoad("data/entities/projectiles/repelling_field_smaller.xml", x, y)

local field_x, field_y = EntityGetTransform(field_id)


local projectiles = EntityGetInRadiusWithTag(field_x, field_y, 25, "projectile")
local creatures = EntityGetInRadiusWithTag(field_x, field_y, 25, "human")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			--GamePrint("enemy projectile with id " .. projectile_id .. " collided with field")
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			for i, component in ipairs(EntityGetComponent(projectile_id, "VariableStorageComponent") or {}) do
				if ComponentGetValue2(component, "name") == "deflected" then
					do return end
				end
			end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("data/entities/particles/particle_explosion/explosion_flare_green_small.xml", x, y)
			EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)

			local projectile_vel_x, projectile_vel_y = 0
			edit_component( projectile_id, "VelocityComponent", function(comp,vars)
				projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
			end)

			-- Inverts projectile velocity, turning it back to the direction it came
			comp_ids = EntityGetComponent( projectile_id, "VelocityComponent" )

			if( comp_ids ~= nil ) then
				--ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
				ComponentSetValueVector2( comp_ids[1], "mVelocity", (projectile_vel_x * 1.5) * -1, (projectile_vel_y * 1.5) * -1)
			end

			-- Sets the shooter of the deflected projectile as the player, so it can damage the creature who shot it originally
			comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )

			if( comp_ids ~= nil ) then
				ComponentSetValue( comp_ids[1], "mWhoShot", entity_id)
				ComponentSetValue( comp_ids[1], "mShooterHerdId", getSpeciesFromEnemyEntity(entity_id))
			end

			-- Flags projectile as deflected to prevent it from changing again
			EntityAddComponent2(projectile_id, "VariableStorageComponent", {
				name = "deflected"
			})
			
		end
	end
end

-- knocks creatures back
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= entity_id) then -- only knocks back creatures other than the entity under this effect
		-- First obtains velocity of entity under this effect
		local entity_vel_x, entity_vel_y = 0
		edit_component( entity_id, "VelocityComponent", function(comp,vars)
			entity_vel_x, entity_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
		end)
		
		-- then applies the same velocity to creature caught within the field
		comp_ids = EntityGetComponent( creature_id, "VelocityComponent" )
		if( comp_ids ~= nil ) then
			ComponentSetValueVector2( comp_ids[1], "mVelocity", entity_vel_x * 10, entity_vel_y * 10)
		end
		
		-- Loads audiovisual feedback
		GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
		EntityLoad("data/entities/particles/particle_explosion/explosion_flare_green_small.xml", x, y)
		EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)
		
		-- Deals damage to creature within field
		local damageThreshold = 0.03 * applyTrinketsEffects_SkullFlask()
		
		EntityInflictDamage( creature_id, damageThreshold, "DAMAGE_MELEE", "knocked back", "NORMAL", 0, 0)
	end	
end



