dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_generic_functions.lua")

local entity_id    = GetUpdatedEntityID()
entity_id = EntityGetRootEntity( entity_id )
local x, y = EntityGetTransform(entity_id)
local field_id = EntityLoad("data/entities/projectiles/scorching_field_smaller.xml", x, y)

-- Sets the shooter of the field as the creature under the effect, so that the effect added by the field itself can harm other creatures
comp_ids = EntityGetComponent( field_id, "ProjectileComponent" )

if( comp_ids ~= nil ) then
	ComponentSetValue( comp_ids[1], "mWhoShot", entity_id)
	ComponentSetValue( comp_ids[1], "mShooterHerdId", getSpeciesFromEnemyEntity(entity_id))
end

local field_x, field_y = EntityGetTransform(field_id)


local projectiles = EntityGetInRadiusWithTag(field_x, field_y, 25, "projectile")
local creatures = EntityGetInRadiusWithTag(field_x, field_y, 25, "human")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= entity_id then -- projectiles are only deflected if not shot by the entity under the effect, this check is necessary otherwise the field itself will be caught within its own field
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			for i, component in ipairs(EntityGetComponent(projectile_id, "VariableStorageComponent") or {}) do
				if ComponentGetValue2(component, "name") == "deflected" then
					do return end
				end
			end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("data/entities/particles/particle_explosion/explosion_flare_green_small.xml", x, y)
			EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)

			local projectile_vel_x, projectile_vel_y = 0
			edit_component( projectile_id, "VelocityComponent", function(comp,vars)
				projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
			end)
			
			-- strips projectile from its ability to explode on death, then kills it and loads firebomb
			edit_component( projectile_id, "ProjectileComponent", function(comp,vars)
				vars.on_death_explode=false
			end)
			EntityKill(projectile_id)
			
			local firebomb_projectile_id = EntityLoad( "data/entities/projectiles/grenade_custom.xml", x, y)

			-- Inverts projectile velocity, turning it back to the direction it came
			comp_ids = EntityGetComponent( firebomb_projectile_id, "VelocityComponent" )

			if( comp_ids ~= nil ) then
				--ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
				ComponentSetValueVector2( comp_ids[1], "mVelocity", (projectile_vel_x * 3) * -1, (projectile_vel_y * 3) * -1)
			end

			-- Sets the shooter of the deflected projectile as the creature under the effect
			comp_ids = EntityGetComponent( firebomb_projectile_id, "ProjectileComponent" )

			if( comp_ids ~= nil ) then
				ComponentSetValue( comp_ids[1], "mWhoShot", entity_id)
				ComponentSetValue( comp_ids[1], "mShooterHerdId", getSpeciesFromEnemyEntity(entity_id))
			end

			-- Flags projectile as deflected to prevent it from changing again
			EntityAddComponent2(firebomb_projectile_id, "VariableStorageComponent", {
				name = "deflected"
			})
			
		end
	end
end