dofile_once("data/scripts/game_helpers.lua" )
dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_utilities.lua")
dofile_once("mods/TheArmory/files/wands_list.lua")
dofile_once("mods/TheArmory/files/containers_list.lua")
dofile_once("mods/TheArmory/files/smallpickups_list.lua")
dofile_once("mods/TheArmory/files/trinkets_list.lua")

function open_chest(entity_id)
	
	local internalName = EntityGetName(entity_id)
	local x, y = EntityGetTransform( entity_id )
	
	SetRandomSeed( GameGetFrameNum(), x + y + entity_id )
	
	local spawn_coordinate_x = x + Random(-10,10)
	local spawn_coordinate_y = y - 4 + Random(-10,10)
	
	local chestOpenned = false
	local openMessage = ""
	
	if(internalName == "chest_armory_regular") then -- Regular Tier Chest - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("regular"), spawn_coordinate_x, spawn_coordinate_y)	
		openMessage = "Regular Chest opened! Regular tier weapon dropped"
		chestOpenned = true
	elseif(internalName == "chest_armory_exquisite") then -- Exquisite Tier Chest - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("exquisite"), spawn_coordinate_x, spawn_coordinate_y)
		openMessage = "Exquisite Chest opened! Exquisite tier weapon dropped"
		chestOpenned = true
	elseif(internalName == "chest_armory_special") then -- Special Tier Chest - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("special"), spawn_coordinate_x, spawn_coordinate_y)
		openMessage = "Special Chest opened! Special tier weapon dropped"
		chestOpenned = true
	elseif(internalName == "chest_armory_rare") then -- Rare Tier Chest - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("rare"), spawn_coordinate_x, spawn_coordinate_y)
		openMessage = "Rare Chest opened! Rare tier weapon dropped"
		chestOpenned = true
	elseif(internalName == "chest_armory_legendary") then -- Legendary Tier Chest - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("legendary"), spawn_coordinate_x, spawn_coordinate_y)
		openMessage = "Legendary Chest opened! Legendary tier weapon dropped"
		chestOpenned = true
	elseif(internalName == "chest_armory_epic") then -- Epic Tier Chest - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("epic"), spawn_coordinate_x, spawn_coordinate_y)
		openMessage = "Epic Chest opened! Epic tier weapon dropped"
		chestOpenned = true
	elseif(internalName == "chest_armory_exotic_unlocked") then -- Exotic Tier Chest (Unlocked) - opens up on interaction, will not open when damaged
		EntityLoad( "data/entities/items/" .. getRandomWand("exotic"), spawn_coordinate_x, spawn_coordinate_y)
		openMessage = "Exotic Chest opened! Exotic tier weapon dropped!"
		chestOpenned = true
	elseif(internalName == "chest_armory_exotic") then -- Exotic Tier Chest - opens up only when interacted with and a chest key is available, will not open when damaged
		if (getPlayerChestKeys() >= 1) then
			addPlayerKeys(-1, "chestkeys")
			EntityLoad( "data/entities/items/" .. getRandomWand("exotic"), spawn_coordinate_x, spawn_coordinate_y)
			openMessage = "You unlocked an Exotic Chest, random Exotic tier weapon dropped!"
			chestOpenned = true
		else
			openMessage = "You need a Chest Key to open this Exotic chest"
			chestOpenned = false
		end
	elseif(internalName == "chest_armory_miniboss") then -- Miniboss Chest - opens up only when interacted with and a miniboss key is available, will not open when damaged
		if (getPlayerMinibossKeys() >= 1) then
			addPlayerKeys(-1, "minibosskeys")
			EntityLoad( "data/entities/items/boss_wands/" .. getRandomMinibossWand(), spawn_coordinate_x, spawn_coordinate_y)
			openMessage = "You unlocked a Miniboss Chest, random Miniboss weapon dropped!"
			chestOpenned = true
		else
			openMessage = "You need a Miniboss Key to open this Miniboss chest"
			chestOpenned = false
		end
	elseif(internalName == "chest_armory_trinkets") then -- Trinkets Chest - opens up only when interacted with and a chest key is available, will not open when damaged
		if (getPlayerChestKeys() >= 1) then
			addPlayerKeys(-1, "chestkeys")
			EntityLoad( getTrinketsBaseDir() .. getRandomTrinket() .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
			openMessage = "You unlocked a Trinkets Chest, random trinket dropped!"
			chestOpenned = true
		else
			openMessage = "You need a Chest Key to open this Trinkets chest"
			chestOpenned = false
		end
	elseif(internalName == "safe") then -- Safe - opens up only when interacted with and a safe key is available, will not open when damaged
		if (getPlayerSafeKeys() >= 1) then
			addPlayerKeys(-1, "safekeys")
			
			local r = Random(1,6)

			if(r == 1) then
				EntityLoad( "data/entities/items/pickup/goldnugget_50.xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/pickup/goldnugget_50.xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/pickup/goldnugget_50.xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/pickup/goldnugget_200.xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/pickup/goldnugget_200.xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/pickup/goldnugget_200.xml", spawn_coordinate_x, spawn_coordinate_y)
			elseif(r == 2) then
				EntityLoad( getContainersBaseDir() .. getRandomTieredChest(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getContainersBaseDir() .. getRandomTieredChest(), spawn_coordinate_x, spawn_coordinate_y)
			elseif(r == 3) then
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("slot_machine_chip").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("slot_machine_chip").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("slot_machine_chip").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("slot_machine_chip").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("slot_machine_chip").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
			elseif(r == 4) then
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), spawn_coordinate_x, spawn_coordinate_y)
			elseif(r == 5) then
				EntityLoad( "data/entities/items/" .. getRandomWand(), spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/" .. getRandomWand(), spawn_coordinate_x + 10, spawn_coordinate_y)
			elseif(r == 6) then
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("medium_heart_powerful").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("medium_heart_powerful").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getPickupItem("medium_heart_powerful").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( "data/entities/items/pickup/milk_bottle.xml", spawn_coordinate_x, spawn_coordinate_y)
			end
			
			openMessage = "You unlocked a Safe, random reward dropped!"
			chestOpenned = true
		else
			openMessage = "You need a Safe Key to open this Safe"
			chestOpenned = false
		
		end
	elseif(internalName == "chest_armory_rainbow") then -- Rainbow Chest, spawns up to four different items
		local maximum_items = 4 --Random(1,4) -- sets random amount of items spawned
		for i=1,maximum_items do
			local item_type = Random(1,3) -- sets random item type, where 1 - regular weapon, 2 - boss weapon, 3 - trinket
			if(item_type == 1) then -- regular weapon
				EntityLoad( getWandsBaseDir() .. getRandomWand(), spawn_coordinate_x + Random(-5, 5), spawn_coordinate_y)
			elseif(item_type == 2) then -- boss weapon
				EntityLoad( getBossWandsBaseDir() .. getRandomMinibossWand(), spawn_coordinate_x + Random(-5, 5), spawn_coordinate_y)
			elseif(item_type == 3) then -- trinket
				EntityLoad( getTrinketsBaseDir() .. getRandomTrinket() .. ".xml", spawn_coordinate_x + Random(-5, 5), spawn_coordinate_y)
			end
		end
		
		openMessage = "Rainbow Chest opened! Random items dropped"
		chestOpenned = true
	end
	
	GamePrint(openMessage)
	GamePrintImportant(openMessage, "" )
	if(chestOpenned) then
		if(internalName == "chest_armory_exotic") then
			EntityLoad("data/entities/particles/image_emitters/chest_effect_blue.xml", x, y)
		else
			EntityLoad("data/entities/particles/image_emitters/chest_effect.xml", x, y)
		end
		EntityKill( entity_id )
	else
		EntityLoad("data/entities/particles/image_emitters/chest_effect_bad.xml", x, y)
		EntityKill (entity_id)
		EntityLoad(getContainersBaseDir() .. internalName .. ".xml", x, y)
	end
end

function item_pickup( entity_item, entity_who_picked, item_name )

	open_chest(entity_item)
	
end

function physics_body_modified( is_destroyed )
	
	open_chest(GetUpdatedEntityID())
end