dofile_once("data/scripts/game_helpers.lua" )
dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_utilities.lua")
-- dofile_once("data/scripts/thearmory_generic_functions.lua") <-- already included within thearmory_utilities file
dofile_once("mods/TheArmory/files/wands_list.lua")
dofile_once("mods/TheArmory/files/containers_list.lua")
dofile_once("mods/TheArmory/files/smallpickups_list.lua")
-- dofile_once("mods/TheArmory/files/recipes_list.lua") <-- no longer used by the crafting system

-- Initial Vending Machine price
local vendingMachineCurrentPrice = 200
-- Default Ammo Vending Machine price
local ammoPrice = 100
-- Default Medical Supplies Vending Machine
local medicalSuppliesPrice = 200
-- Default health threshold used by blood donation machines
local bloodMachineThreshold = 0.8
-- Variables used by the health for ammo machine
local minimum_ammo_types_count = 5
local individual_ammo_count_threshold = 10
-- Threshold for health percentage, used by the health for ammo machine
local current_health_threshold = 75
-- Gold threshold used by the health for gold machine
local minimum_gold_threshold = 50
-- Key threshold used by the health for keys machine
local minimum_keys_threshold = 2

-- Default dir for all interactive machines
local interactiveMachinesBaseDir = "data/entities/items/pickup/"

function machine_activate(entity_id)

	local internalName = EntityGetName(entity_id)
	local x, y = EntityGetTransform( entity_id )
	
	SetRandomSeed( GameGetFrameNum(), x + y + entity_id )
	
	local interactionSuccesful = false
	local interactionMessage = ""
	
	-- for interactive machines that spawn items, determines whether they must spawn right or left from the vending machine
	-- this is done to avoid machines that accept items to be sold to automatically consume the item
	local player_x, player_y = EntityGetTransform(getPlayerEntity())
	local machine_x, machine_y = EntityGetTransform(entity_id)
	
	local item_spawn_offset_x = 0
	
	if(machine_x >= player_x) then
		item_spawn_offset_x = 20 * -1
	else
		item_spawn_offset_x = 20
	end
	
	local spawn_coordinate_x = player_x + item_spawn_offset_x
	local spawn_coordinate_y = player_y + 2
	
	
	if(internalName == "vending_machine") then -- Regular Vending Machine
		
		-- Verifies first if the player has the current amount of gold required
		if (getPlayerGold() >= vendingMachineCurrentPrice) then
			
			-- Deducts gold from player
			addPlayerGold(vendingMachineCurrentPrice * -1)
			
			-- Spawns random wand
			EntityLoad( "data/entities/items/" .. getRandomWand(), spawn_coordinate_x, spawn_coordinate_y)
			
			-- Raises price
			local vendingMachineNewPrice = vendingMachineCurrentPrice + 200
			
			interactionMessage = "Random weapon bought for " .. vendingMachineCurrentPrice .. " gold! New price is " .. vendingMachineNewPrice .. " gold"
			
			-- Updates price
			vendingMachineCurrentPrice = vendingMachineNewPrice
			
			interactionSuccesful = true
		else
			interactionMessage = vendingMachineCurrentPrice .. " Gold is required to buy a new weapon"
			interactionSuccesful = false
		end
	
	elseif(internalName == "ammo_vending_machine") then -- Ammo Vending Machine
		
		-- Verifies first if the player has the current amount of gold required
		if (getPlayerGold() >= ammoPrice) then
			
			-- Deducts gold from player
			addPlayerGold(ammoPrice * -1)
			
			-- Always spawns regular bullets ammo box
			EntityLoad( getContainersBaseDir() .. "bullet_ammo_box.xml", spawn_coordinate_x, spawn_coordinate_y)
			-- Chance to spawn an additional ammo box
			local r = Random(1,2)
			if(r == 1) then
				EntityLoad( getContainersBaseDir() .. getRandomAmmoBox(), x, y)
			end
		
			interactionMessage = "You bought an ammo box!"
			interactionSuccesful = true
		else
			interactionMessage = ammoPrice .. " Gold is required to buy an ammo box"
			interactionSuccesful = false
		end
	
	elseif(internalName == "medical_supplies_vending_machine") then -- Medical Supplies Vending Machine
		
		-- Verifies first if the player has the current amount of gold required
		if (getPlayerGold() >= medicalSuppliesPrice) then
			
			-- Deducts gold from player
			addPlayerGold(medicalSuppliesPrice * -1)
			
			local r = Random(1,3)
			
			-- Can spawn first aid boxes, milk bottles or regular or powerful hearts
			if( r == 1 ) then
				EntityLoad( getContainersBaseDir() .. "first_aid_box.xml", spawn_coordinate_x, spawn_coordinate_y)	
			elseif( r == 2 ) then
				EntityLoad( getPickupItemsBaseDir() .. getRandomPickupItem("healing", "positive").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getRandomPickupItem("healing", "positive").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getRandomPickupItem("healing", "positive").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				EntityLoad( getPickupItemsBaseDir() .. getRandomPickupItem("healing", "positive").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
			elseif( r == 3 ) then
				local r_milk_bottle = Random(1,4)
				if(r_milk_bottle == 1) then
					EntityLoad("data/entities/items/one_time_use/milk_bottle.xml", spawn_coordinate_x, spawn_coordinate_y)
				else
					EntityLoad( getPickupItemsBaseDir() .. getRandomPickupItem("healing", "powerful").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
					EntityLoad( getPickupItemsBaseDir() .. getRandomPickupItem("healing", "powerful").name .. ".xml", spawn_coordinate_x, spawn_coordinate_y)
				end
			end	
			
	
			interactionMessage = "You bought a healing item!"
			interactionSuccesful = true
		else
			interactionMessage = medicalSuppliesPrice .. " Gold is required to buy a new healing item"
			interactionSuccesful = false
		end
	
	elseif(internalName == "slot_machine") then -- Slot Machine
		
		-- Verifies first if the player has a slot machine chip
		if (getPlayerSlotMachineChips() >= 1) then
		
			-- Deducts slot machine chip from player
			addPlayerSlotMachineChips(-1)
			
			-- Spawns random tiered chest
			EntityLoad( getContainersBaseDir() .. getRandomTieredChest(), spawn_coordinate_x, spawn_coordinate_y)	
			
			interactionMessage = "You tried your luck on the slot machine!"
			interactionSuccesful = true
		else
			interactionMessage = "You don't have a slot machine chip"
			interactionSuccesful = false
		end
	
	elseif(internalName == "justice_machine") then -- Justice Machine
		
		-- Verifies first if the player has any enemy guts
		local playerEnemyGuts = getPlayerEnemyGuts()
		if(playerEnemyGuts >= 1) then
				
			-- Adds firearms ammo, proportional to the amount of enemy guts deposited
			rifleAmmo 		= getPickupItem("bullet_rifle_ammo")
			shotgunAmmo 	= getPickupItem("bullet_shotgun_ammo")
			smgAmmo 		= getPickupItem("bullet_smg_ammo")
			arAmmo			= getPickupItem("bullet_ar_rifle_ammo")
			pistolAmmo 		= getPickupItem("bullet_pistol_ammo")
			revolverAmmo	= getPickupItem("bullet_revolver_ammo")

			addPlayerAmmo(playerEnemyGuts * 5, rifleAmmo.related_variable, rifleAmmo.ui_name)
			addPlayerAmmo(playerEnemyGuts * 2, shotgunAmmo.related_variable, shotgunAmmo.ui_name)
			addPlayerAmmo(playerEnemyGuts * 5, smgAmmo.related_variable, smgAmmo.ui_name)
			addPlayerAmmo(playerEnemyGuts * 5, arAmmo.related_variable, arAmmo.ui_name)
			addPlayerAmmo(playerEnemyGuts * 5, pistolAmmo.related_variable, pistolAmmo.ui_name)
			addPlayerAmmo(playerEnemyGuts * 2, revolverAmmo.related_variable, revolverAmmo.ui_name)
			
			-- Adds health and gold, proportional to the amount of enemy guts deposited
			addPlayerHealth(10 * playerEnemyGuts, false) -- does not overheal if max health was reached
			addPlayerGold(10 * playerEnemyGuts)
			
			-- Deducts all enemy guts deposited from player
			addPlayerEnemyGuts(playerEnemyGuts * -1)
			
			interactionMessage = "You have done justice and have been rewarded with ammo, health and gold!"
			interactionSuccesful = true
		else
			interactionMessage = "You don't have enemy guts to deposit"
			interactionSuccesful = false
		end
		
	elseif(internalName == "blood_machine_ammo") then -- Blood Machine (Ammo)
		
		-- First, checks if the player is under the REGENERATION effect, refuses service if true
		if(GameGetGameEffectCount(getPlayerEntity(), "REGENERATION") > 0) then
			interactionMessage = "Service refused, you are currently being healed"
			interactionSuccesful = false
		else
			-- Obtains player health
			local playerHealth = getPlayerHealth()
			if(playerHealth <= bloodMachineThreshold) then -- Equivalent to around 12 health
				interactionMessage = "You are in no condition to give up 50% Health"
				interactionSuccesful = false
			else
				-- Halves player total health
				playerHealth = playerHealth / 2
				-- Sets new player health
				setPlayerHealth(playerHealth)
				
				-- Gives fixed ammounts of ammo per interaction
				rifleAmmo 		= getPickupItem("bullet_rifle_ammo")
				shotgunAmmo 	= getPickupItem("bullet_shotgun_ammo")
				smgAmmo 		= getPickupItem("bullet_smg_ammo")
				arAmmo			= getPickupItem("bullet_ar_rifle_ammo")
				pistolAmmo 		= getPickupItem("bullet_pistol_ammo")
				revolverAmmo	= getPickupItem("bullet_revolver_ammo")
				
				addPlayerAmmo(30, rifleAmmo.related_variable, rifleAmmo.ui_name)
				addPlayerAmmo(15, shotgunAmmo.related_variable, shotgunAmmo.ui_name)
				addPlayerAmmo(45, smgAmmo.related_variable, smgAmmo.ui_name)
				addPlayerAmmo(30, arAmmo.related_variable, arAmmo.ui_name)
				addPlayerAmmo(30, pistolAmmo.related_variable, pistolAmmo.ui_name)
				addPlayerAmmo(30, revolverAmmo.related_variable, revolverAmmo.ui_name)
				
				interactionMessage = "You gave up %50 Health for firearms ammo"
				interactionSuccesful = true
			end	
		end
		
	elseif(internalName == "blood_machine_gold") then -- Blood Machine (Gold)
		
		-- First, checks if the player is under the REGENERATION effect, refuses service if true
		if(GameGetGameEffectCount(getPlayerEntity(), "REGENERATION") > 0) then
			interactionMessage = "Service refused, you are currently being healed"
			interactionSuccesful = false
		else
			-- Obtains player health
			local playerHealth = getPlayerHealth()
			if(playerHealth <= bloodMachineThreshold) then -- Equivalent to around 12 health
				interactionMessage = "You are in no condition to give up 50% Health"
				interactionSuccesful = false
			else
				-- Halves player total health
				playerHealth = playerHealth / 2
				-- Sets new player health
				setPlayerHealth(playerHealth)
				-- Adds gold
				addPlayerGold(200)
					
				interactionMessage = "You gave up %50 Health for 200 gold"
				interactionSuccesful = true
			end
		end
	
	elseif(internalName == "blood_machine_gun") then -- Blood Machine (Gun)
		
		-- First, checks if the player is under the REGENERATION effect, refuses service if true
		if(GameGetGameEffectCount(getPlayerEntity(), "REGENERATION") > 0) then
			interactionMessage = "Service refused, you are currently being healed"
			interactionSuccesful = false
		else
		
			-- Obtains player health
			local playerHealth = getPlayerHealth()
			if (playerHealth <= bloodMachineThreshold) then -- Equivalent to around 12 health	
				interactionMessage = "You are in no condition to give up 50% Health"
				interactionSuccesful = false
			else
				-- Halves player total health
				playerHealth = playerHealth / 2
				-- Sets new player health
				setPlayerHealth(playerHealth)
				-- Spawns random wand
				EntityLoad( "data/entities/items/" .. getRandomWand(), spawn_coordinate_x, spawn_coordinate_y)
				
				interactionMessage = "You gave up %50 Health for a random weapon!"
				interactionSuccesful = true
			end
		end
		
	elseif(internalName == "blood_machine_keys") then -- Blood Machine (Keys)
	
		-- First, checks if the player is under the REGENERATION effect, refuses service if true
		if(GameGetGameEffectCount(getPlayerEntity(), "REGENERATION") > 0) then
			interactionMessage = "Service refused, you are currently being healed"
			interactionSuccesful = false
		else
			-- Obtains player health
			local playerHealth = getPlayerHealth()
			if (playerHealth <= bloodMachineThreshold) then -- Equivalent to around 12 health	
				interactionMessage = "You are in no condition to give up 50% Health"
				interactionSuccesful = false
			else
				-- Halves player total health
				playerHealth = playerHealth / 2
				-- Sets new player health
				setPlayerHealth(playerHealth)
				
				chest_key 		= getPickupItem("chest_key")
				safe_key 		= getPickupItem("safe_key")
				
				addPlayerKeys(chest_key.base_value, chest_key.related_variable)
				addPlayerKeys(safe_key.base_value, safe_key.related_variable)
				
				interactionMessage = "You gave up %50 Health for one chest key and one safe key!"
				interactionSuccesful = true
			end
		end
		
	elseif(internalName == "health_for_ammo_machine") then -- Health for Ammo Machine
		-- First, calculates the percentage of current health compared to max health, and refuses service if more than 75%
		if(getPlayerCurrentHealthPercentage() > current_health_threshold) then
			interactionMessage = "Service refused, you have more than " .. current_health_threshold .. "% of your total health"
			interactionSuccesful = false
		else
			-- Obtains current ammo types table and elements count
			local ammo_types = getAmmoTypes()
			local ammo_types_count = getAmmoTypesCount()
			
			-- list to store eligible ammo types to be modified
			local eligible_ammo_types = {}
			
			-- counter for eligible ammo types
			local eligible_ammo_types_count = 0
			
			-- for each ammo counter that is above 10, adds it to eligible_ammo_types list
			for i,element in ipairs(ammo_types) do
				if(getPlayerAmmo(element) >= individual_ammo_count_threshold) then
					table.insert(eligible_ammo_types, element)
					--GamePrint(element .. " is above " .. individual_ammo_count_threshold)
					eligible_ammo_types_count = eligible_ammo_types_count + 1
				end
			end
			
			if(eligible_ammo_types_count < minimum_ammo_types_count) then -- if less than minimum_ammo_types_count ammo counters are above 10, service is refused
				interactionMessage = "At least " .. minimum_ammo_types_count .. " ammo types must have counts above " .. individual_ammo_count_threshold .. " to use this machine"
				interactionSuccesful = false
			else
				for i,element in ipairs(eligible_ammo_types) do
					-- deducts 50% of each eligible ammo type
					local ammo_deduct = (getPlayerAmmo(element) / 2) * -1
					addPlayerAmmo(ammo_deduct, element, "")
				end	
				
				addPlayerHealthByPercentage(50)
				
				interactionMessage = "You gave up 50% of your ammo with counts above " .. individual_ammo_count_threshold.. " for 50% health bonus"
				interactionSuccesful = true
			end
		end
		
	elseif(internalName == "health_for_gold_machine") then -- Health for Gold Machine
		-- First, calculates the percentage of current health compared to max health, and refuses service if more than 75%
		if(getPlayerCurrentHealthPercentage() > current_health_threshold) then
			interactionMessage = "Service refused, you have more than " .. current_health_threshold .. "% of your total health"
			interactionSuccesful = false
		else
			if(getPlayerGold() < minimum_gold_threshold) then
				interactionMessage = "At least " .. minimum_gold_threshold .. " gold is required to use this machine"
				interactionSuccesful = false
			else
				local gold_deduct = math.floor(getPlayerGold() * 50 / 100) * -1
				addPlayerGold(gold_deduct)
				
				addPlayerHealthByPercentage(50)
				
				interactionMessage = "You gave up 50% of your gold for 50% health bonus"
				interactionSuccesful = true
			end
		end
		
	elseif(internalName == "health_for_keys_machine") then -- Health for Keys Machine
		-- First, calculates the percentage of current health compared to max health, and refuses service if more than 75%
		if(getPlayerCurrentHealthPercentage() > current_health_threshold) then
			interactionMessage = "Service refused, you have more than " .. current_health_threshold .. "% of your total health"
			interactionSuccesful = false
		else
			if(not (getPlayerChestKeys() >= minimum_keys_threshold and getPlayerSafeKeys() >= minimum_keys_threshold)) then
				interactionMessage = "At least " .. minimum_keys_threshold .. " safe and chest keys are required to use this machine"
				interactionSuccesful = false
			else
				local chest_keys_deduct = math.floor(getPlayerChestKeys() * 50 / 100) * -1
				local safe_keys_deduct = math.floor(getPlayerSafeKeys() * 50 / 100) * -1
				
				chest_key 		= getPickupItem("chest_key")
				safe_key 		= getPickupItem("safe_key")
				
				addPlayerKeys(chest_keys_deduct, chest_key.related_variable)
				addPlayerKeys(safe_keys_deduct, safe_key.related_variable)
				
				addPlayerHealthByPercentage(50)
				
				interactionMessage = "You gave up 50% of your chest and safe keys for 50% health bonus"
				interactionSuccesful = true
			end
		end
		
	elseif(internalName == "health_for_gun_machine") then -- Health for Gun Machine
		-- First, calculates the percentage of current health compared to max health, and refuses service if more than 75%
		if(getPlayerCurrentHealthPercentage() > current_health_threshold) then
			interactionMessage = "Service refused, you have more than " .. current_health_threshold .. "% of your total health"
			interactionSuccesful = false
		else
			if(not isCurrentWandTheArmory()) then
				interactionMessage = "This machine accepts only weapons dropped from chests or minibosses"
				interactionSuccesful = false
			else
				
				EntityKill(getCurrentlyEquippedWandId())
				
				addPlayerHealthByPercentage(50)
				
				interactionMessage = "You gave up your currently equipped weapon for 50% health bonus"
				interactionSuccesful = true
			end
		end
	
	elseif(internalName == "furnace") then -- Furnace
	
		local oresSmelted = 0
		
		local ore_iron = getPlayerCraftingParts("ore_iron")
		local ore_copper = getPlayerCraftingParts("ore_copper")
		local ore_gold = getPlayerCraftingParts("ore_gold")
		local ore_jade = getPlayerCraftingParts("ore_jade")
		local ore_amethyst = getPlayerCraftingParts("ore_amethyst")
		local ore_diamond = getPlayerCraftingParts("ore_diamond")
		local ore_ruby = getPlayerCraftingParts("ore_ruby")
		local ore_purple_fluorite = getPlayerCraftingParts("ore_purple_fluorite")
		
		if(ore_iron ~= 0) then
			for i=1,ore_iron do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_iron")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_iron * -1, "ore_iron")
		end
		
		if(ore_copper ~= 0) then
			for i=1,ore_copper do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_copper")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_copper * -1, "ore_copper")
		end
		
		if(ore_gold ~= 0) then
			for i=1,ore_gold do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_gold")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_gold * -1, "ore_gold")
		end
		
		if(ore_jade ~= 0) then
			for i=1,ore_jade do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_jade")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_jade * -1, "ore_jade")
		end
		
		if(ore_amethyst ~= 0) then
			for i=1,ore_amethyst do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_amethyst")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_amethyst * -1, "ore_amethyst")
		end
		
		if(ore_diamond ~= 0) then
			for i=1,ore_diamond do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_diamond")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_diamond * -1, "ore_diamond")
		end
		
		if(ore_ruby ~= 0) then
			for i=1,ore_ruby do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_ruby")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_ruby * -1, "ore_ruby")
		end
		
		if(ore_purple_fluorite ~= 0) then
			for i=1,ore_purple_fluorite do
				-- adds ingots according to ore amount
				addPlayerCraftingParts(1, "ingot_purple_fluorite")
				oresSmelted = oresSmelted + 1
			end
			-- deducts ore from player
			addPlayerCraftingParts(ore_purple_fluorite * -1, "ore_purple_fluorite")
		end
		
		if (oresSmelted > 0) then
			interactionMessage = "Ingots produced and added to your inventory"
			interactionSuccesful = true
		else
			interactionMessage = "You don't have any ore to smelt"
			interactionSuccesful = false
		end
		
	end
	
	GamePrint(interactionMessage)
	GamePrintImportant(interactionMessage, "" )
	if(interactionSuccesful) then
		if(internalName == "slot_machine") then
			EntityLoad("data/entities/particles/image_emitters/chest_effect_blue.xml", x, y)
		elseif(internalName == "justice_machine") then
			EntityLoad("data/entities/particles/image_emitters/chest_effect_green.xml", x, y)
		else
			EntityLoad("data/entities/particles/image_emitters/chest_effect.xml", x, y)
		end
		
		EntityKill (entity_id)
		EntityLoad(interactiveMachinesBaseDir .. internalName .. ".xml", x, y)
	else
		EntityLoad("data/entities/particles/image_emitters/chest_effect_bad.xml", x, y)
		EntityKill (entity_id)
		EntityLoad(interactiveMachinesBaseDir .. internalName .. ".xml", x, y)
	end

end

function item_pickup( entity_item, entity_who_picked, item_name )

	machine_activate(entity_item)
	
end