dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_utilities.lua")

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local projectiles = EntityGetInRadiusWithTag(x, y, 40, "projectile")
local creatures = EntityGetInRadiusWithTag(x, y, 40, "enemy")

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= getPlayerEntity() then -- projectiles are only deflected if not shot by the player
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			for i, component in ipairs(EntityGetComponent(projectile_id, "VariableStorageComponent") or {}) do
				if ComponentGetValue2(component, "name") == "deflected" then
					do return end
				end
			end
			
			-- Then, checks if the projectile was shot by the shadow wizard, returns to the caller if so
			-- projectiles show by this creature are not affected by the field
			local shadowwizard_id = EntityGetWithName( "shadowwizard.xml" )
			comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )
			if( comp_ids ~= nil ) then
				if(ComponentGetValue2( comp_ids[1], "mWhoShot") == shadowwizard_id) then
					do return end
				end
			end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("data/entities/particles/particle_explosion/explosion_flare_green_small.xml", x, y)
			EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)

			local projectile_vel_x, projectile_vel_y = 0
			edit_component( projectile_id, "VelocityComponent", function(comp,vars)
				projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
			end)

			-- Inverts projectile velocity, turning it back to the direction it came
			comp_ids = EntityGetComponent( projectile_id, "VelocityComponent" )

			if( comp_ids ~= nil ) then
				--ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
				ComponentSetValueVector2( comp_ids[1], "mVelocity", (projectile_vel_x * 1.5) * -1, (projectile_vel_y * 1.5) * -1)
			end

			-- Sets the shooter of the deflected projectile as the player
			comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )

			if( comp_ids ~= nil ) then
				ComponentSetValue( comp_ids[1], "mWhoShot", getPlayerEntity())
				ComponentSetValue( comp_ids[1], "mShooterHerdId", getSpeciesFromEnemyEntity(getPlayerEntity()))
			end

			-- Flags projectile as deflected to prevent it from changing again
			EntityAddComponent2(projectile_id, "VariableStorageComponent", {
				name = "deflected"
			})
			
		end
	end
end

-- damages creatures
for i, creature_id in ipairs(creatures) do
	if(creature_id ~= getPlayerEntity()) then -- only damages creatures other than the player
		
		-- Loads audiovisual feedback
		GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
		EntityLoad("data/entities/particles/particle_explosion/explosion_flare_green_small.xml", x, y)
		EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)
		
		-- Deals damage to creature within field
		EntityInflictDamage( creature_id, 0.03, "DAMAGE_MELEE", "knocked back", "NORMAL", 0, 0)
	end	
end



