dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_generic_functions.lua")

function collision_trigger(colliding_entity_id)

	local x, y = EntityGetTransform(colliding_entity_id)

	-- First checks if the projectile has already been deflected, returns to the caller if so
	for i, component in ipairs(EntityGetComponent(colliding_entity_id, "VariableStorageComponent") or {}) do
		if ComponentGetValue2(component, "name") == "deflected" then
			do return end
		end
	end
	
	-- Loads audiovisual feedback
	GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
	EntityLoad("data/entities/particles/particle_explosion/explosion_flare_small.xml", x, y)
	EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)
	
	local projectile_vel_x, projectile_vel_y = 0
	edit_component( colliding_entity_id, "VelocityComponent", function(comp,vars)
		projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
	end)
	
	-- strips projectile from its ability to explode on death, then kills it
	edit_component( colliding_entity_id, "ProjectileComponent", function(comp,vars)
		vars.on_death_explode=false
	end)
	EntityKill(colliding_entity_id)
			
	local thunderbolt_projectile_id = EntityLoad( "data/entities/projectiles/drillbomb.xml", x, y)
	
	-- Deflects enemy projectile at either a negative or positive angle, or straight
	SetRandomSeed( GameGetFrameNum(), x + y + thunderbolt_projectile_id)
	
	local angle = 0 - math.atan2( projectile_vel_y * -1, projectile_vel_x * -1 )
	
	comp_ids = EntityGetComponent( thunderbolt_projectile_id, "VelocityComponent" )

	if( comp_ids ~= nil ) then
		ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
		ComponentSetValueVector2( comp_ids[1], "mVelocity", (projectile_vel_x * 2.5) * -1, (projectile_vel_y * 2.5) * -1)
	end

	-- Sets the shooter of the deflected projectile as the player, so it can damage the creature who shot it originally
	comp_ids = EntityGetComponent( thunderbolt_projectile_id, "ProjectileComponent" )
	
	if( comp_ids ~= nil ) then
		ComponentSetValue( comp_ids[1], "mWhoShot", getPlayerEntity())
		ComponentSetValue( comp_ids[1], "mShooterHerdId", getSpeciesFromEnemyEntity(getPlayerEntity()))
	end
	
	-- Flags projectile as deflected to prevent it from changing again due to this callback
	EntityAddComponent2(thunderbolt_projectile_id, "VariableStorageComponent", {
		name = "deflected"
	})
end