dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_generic_functions.lua")

function collision_trigger(colliding_entity_id)

	local x, y = EntityGetTransform(colliding_entity_id)

	-- First checks if the projectile has already been deflected, returns to the caller if so
	for i, component in ipairs(EntityGetComponent(colliding_entity_id, "VariableStorageComponent") or {}) do
		if ComponentGetValue2(component, "name") == "deflected" then
			do return end
		end
	end
	
	-- Loads audiovisual feedback
	GamePlaySound( "data/audio/Desktop/explosions.snd", "explosions/tnt", x, y )
	EntityLoad("data/entities/particles/particle_explosion/explosion_flare_small.xml", x, y)
	EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)
	
	local projectile_vel_x, projectile_vel_y = 0
	edit_component( colliding_entity_id, "VelocityComponent", function(comp,vars)
		projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
	end)
	
	-- strips projectile from its ability to explode on death, then kills it
	edit_component( colliding_entity_id, "ProjectileComponent", function(comp,vars)
		vars.on_death_explode=false
	end)
	EntityKill(colliding_entity_id)
	
	local vel_x = 0
	local vel_y = 0
	
	SetRandomSeed( GameGetFrameNum(), x + y + colliding_entity_id)

	local angle = 0 - math.atan2( projectile_vel_y * -1, projectile_vel_x * -1 )
	local length = 250

	local angles = {0.05, 0.1, 0.15}
	local angles_count = 3

	-- positive angled shots
	for i=1,2 do
		local angle_up = angle + 3.1415 * angles[Random(1,angles_count)]
		
		vel_x = math.cos( angle_up ) * length
		vel_y = 0 - math.sin( angle_up ) * length
		
		local entity_id_projectile1 = EntityLoad( "data/entities/projectiles/hellfireprojectile.xml", x, y)
		-- Flags projectile as deflected to prevent it from changing again due to this callback
		EntityAddComponent2(entity_id_projectile1, "VariableStorageComponent", {
			name = "deflected"
		})
		comp_ids = EntityGetComponent( entity_id_projectile1, "VelocityComponent" )

		if( comp_ids ~= nil ) then
			ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
			ComponentSetValueVector2( comp_ids[1], "mVelocity", vel_x, vel_y)
		end
	end

	-- straight shot
	local angle_straight = angle + 3.1415 * 0.0001
	vel_x = math.cos( angle_straight ) * length
	vel_y = 0 - math.sin( angle_straight ) * length

	local entity_id_projectile1 = EntityLoad( "data/entities/projectiles/hellfireprojectile.xml", x, y)
	-- Flags projectile as deflected to prevent it from changing again due to this callback
	EntityAddComponent2(entity_id_projectile1, "VariableStorageComponent", {
		name = "deflected"
	})
	comp_ids = EntityGetComponent( entity_id_projectile1, "VelocityComponent" )

	if( comp_ids ~= nil ) then
		ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
		ComponentSetValueVector2( comp_ids[1], "mVelocity", vel_x, vel_y)
	end

	-- negative angled shots
	for i=1,2 do
		local angle_down = angle - 3.1415 * angles[Random(1,angles_count)]
		
		vel_x = math.cos( angle_down ) * length
		vel_y = 0 - math.sin( angle_down ) * length
		
		local entity_id_projectile1 = EntityLoad( "data/entities/projectiles/hellfireprojectile.xml", x, y)
		-- Flags projectile as deflected to prevent it from changing again due to this callback
		EntityAddComponent2(entity_id_projectile1, "VariableStorageComponent", {
			name = "deflected"
		})
		comp_ids = EntityGetComponent( entity_id_projectile1, "VelocityComponent" )

		if( comp_ids ~= nil ) then
			ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
			ComponentSetValueVector2( comp_ids[1], "mVelocity", vel_x, vel_y)
		end
	end
end