dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_utilities.lua")

local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
SetRandomSeed( GameGetFrameNum(), x + y + entity_id )

local projectiles = EntityGetInRadiusWithTag(x, y, 40, "projectile")
local players = EntityGetInRadiusWithTag(x, y, 40, "player_unit")

local shadowwizard_id = EntityGetWithName( "shadowwizard.xml" )

-- deflects projectiles
for i, projectile_id in ipairs(projectiles) do
	for i, component in ipairs(EntityGetComponent(projectile_id, "ProjectileComponent") or {}) do
		if ComponentGetValue2(component, "mWhoShot") ~= shadowwizard_id then -- projectiles are only deflected if not shot by the shadow wizard
			
			local x, y = EntityGetTransform(projectile_id)

			-- First checks if the projectile has already been deflected, returns to the caller if so
			for i, component in ipairs(EntityGetComponent(projectile_id, "VariableStorageComponent") or {}) do
				if ComponentGetValue2(component, "name") == "deflected" then
					do return end
				end
			end
			
			-- Then, checks if the projectile was shot by the shadow wizard, returns to the caller if so
			-- projectiles show by this creature are not affected by the field

			comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )
			if( comp_ids ~= nil ) then
				if(ComponentGetValue2( comp_ids[1], "mWhoShot") == shadowwizard_id) then
					do return end
				end
			end

			-- Loads audiovisual feedback
			GamePlaySound( "data/audio/Desktop/projectiles.snd", "player_projectiles/shield/block", x, y )
			EntityLoad("data/entities/particles/particle_explosion/explosion_flare_green_small.xml", x, y)
			EntityLoad("data/entities/smoke_pocket_smaller.xml", x, y)

			local projectile_vel_x, projectile_vel_y = 0
			edit_component( projectile_id, "VelocityComponent", function(comp,vars)
				projectile_vel_x, projectile_vel_y = ComponentGetValueVector2( comp, "mVelocity", vel_x, vel_y)
			end)

			-- Inverts projectile velocity, turning it back to the direction it came
			comp_ids = EntityGetComponent( projectile_id, "VelocityComponent" )

			if( comp_ids ~= nil ) then
				--ComponentSetValue(comp_ids[1], "mass", Random(4, 6))
				ComponentSetValueVector2( comp_ids[1], "mVelocity", (projectile_vel_x * 1.5) * -1, (projectile_vel_y * 1.5) * -1)
			end

			-- Sets the shooter of the deflected projectile as the shadow wizard
			comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )

			if( comp_ids ~= nil ) then
				ComponentSetValue( comp_ids[1], "mWhoShot", shadowwizard_id)
				ComponentSetValue( comp_ids[1], "mShooterHerdId", getSpeciesFromEnemyEntity(shadowwizard_id))
			end

			-- Flags projectile as deflected to prevent it from changing again
			EntityAddComponent2(projectile_id, "VariableStorageComponent", {
				name = "deflected"
			})
			
		end
	end
end

-- steals gold from the player
for i, player_id in ipairs(players) do
	if(getPlayerGold() >= 10) then
		local player_x, player_y = EntityGetTransform(getPlayerEntity())
		addPlayerGold(-10)
		EntityLoad("data/entities/items/pickup/goldnugget_10.xml", player_x + Random(-30,30), player_y + Random(-30,30))
	end
end



