-- Checks whether or not the player has openned the inventory
function isInventoryOpen()
	local inventory = EntityGetFirstComponent( getPlayerEntity(), "InventoryGuiComponent" );
	local is_inventory_open = false;
	if inventory ~= nil then
		if ComponentGetValue( inventory, "mActive" ) == "1" then
			is_inventory_open = true;
		end
	end
	
	return is_inventory_open
end

-- kills all wands, items and physics objects within supplied radius
function removeAllWandsandItemsInRadius(radius, x, y)
	local vanilla_wands = EntityGetInRadiusWithTag( x, y, radius, "wand" )
	if ( #vanilla_wands > 0 ) then
		for i,vanilla_wand_id in ipairs( vanilla_wands ) do
			if (isObjectDropped(vanilla_wand_id)) then
				-- workaround to prevent the kantele from being removed
				local spriteComponent = EntityGetFirstComponentIncludingDisabled(vanilla_wand_id, "SpriteComponent")
				local sprite_definition_path = ComponentGetValue2(spriteComponent, "image_file")
				if(sprite_definition_path == "data/items_gfx/kantele.xml") then
					--GamePrint("kantele found, ignoring it")
				else
					EntityKill(vanilla_wand_id)
				end
			end
		end
	end
	local thearmory_wands = EntityGetInRadiusWithTag( x, y, radius, "custom_wand" )
	if ( #thearmory_wands > 0 ) then
		for i,thearmory_wand_id in ipairs( thearmory_wands ) do
			if (isObjectDropped(thearmory_wand_id)) then
				EntityKill(thearmory_wand_id)
			end
		end
	end
	local thearmory_miniboss_wands = EntityGetInRadiusWithTag( x, y, radius, "boss_reward_wand" )
	if ( #thearmory_miniboss_wands > 0 ) then
		for i,thearmory_miniboss_wand_id in ipairs( thearmory_miniboss_wands ) do
			if (isObjectDropped(thearmory_miniboss_wand_id)) then
				EntityKill(thearmory_miniboss_wand_id)
			end
		end
	end
	local items = EntityGetInRadiusWithTag( x, y, radius, "item" )
	if ( #items > 0 ) then
		for i,item_id in ipairs( items ) do
			if (isObjectDropped(item_id)) then
				-- workaround to prevent the kantele from being removed
				local spriteComponent = EntityGetFirstComponentIncludingDisabled(item_id, "SpriteComponent")
				local sprite_definition_path = ComponentGetValue2(spriteComponent, "image_file")
				if(sprite_definition_path == "data/items_gfx/kantele.xml") then
					--GamePrint("kantele found, ignoring it")
				else
					EntityKill(item_id)
				end
			end
		end
	end
	local props_physics = EntityGetInRadiusWithTag( x, y, radius, "prop_physics" )
	if ( #props_physics > 0 ) then
		for i,prop_physics_id in ipairs( props_physics ) do
			EntityKill(prop_physics_id)
		end
	end
	local props = EntityGetInRadiusWithTag( x, y, radius, "prop" )
	if ( #props > 0 ) then
		for i,prop_id in ipairs( props ) do
			EntityKill(prop_id)
		end
	end
	local enemies = EntityGetInRadiusWithTag( x, y, radius, "enemy" )
	if ( #enemies > 0 ) then
		for i,enemy_id in ipairs( enemies ) do
			EntityKill(enemy_id)
		end
	end
	local items_physics = EntityGetInRadiusWithTag( x, y, radius, "item_physics" )
	if ( #items_physics > 0 ) then
		for i,item_physics in ipairs( items_physics ) do
			if (isObjectDropped(item_physics)) then
				EntityKill(item_physics)
			end
		end
	end
	
	
end

-- returns true if entity_id contains a script path of callbackType containing scriptName
function entityHasLuaComponentWithScriptName(entity_id, scriptName, callbackType)
	local flag = false
	local components = EntityGetComponent( entity_id, "LuaComponent" )
	for i,component in ipairs(components) do
		if(string.find(ComponentGetValue2( component, callbackType ), scriptName)) then
			flag = true
		end
	end
	return flag
end

-- function that removes a LuaComponent of callbackType that points to script path containing scriptName from an entity whose it is entity_id
function removeLuaComponentWithScriptName(entity_id, scriptName, callbackType)
	local components = EntityGetComponent( entity_id, "LuaComponent" )
	for i,component in ipairs(components) do
		if(string.find(ComponentGetValue2( component, callbackType ), scriptName)) then
			EntityRemoveComponent(entity_id, component)
		end
	end
end

-- adapted version of Goki's Things mod wand locking mechanism
-- it blocks the ability to edit both the wand AND spells bound to it by switching their is_frozen properties to true
function lockWand(wand_id)
	-- First locks all spells/actions bound to the wand
	local children = EntityGetAllChildren( wand_id ) or {};
	for i,action in ipairs( children ) do
		local item = EntityGetFirstComponentIncludingDisabled( action, "ItemComponent" );
		if item ~= nil then
			ComponentSetValue2( item, "is_frozen", true );
		end
	end
	-- Then locks the wand itself
	local item = EntityGetFirstComponentIncludingDisabled( wand_id, "ItemComponent" );
	if item ~= nil then
		ComponentSetValue2( item, "is_frozen", true );
	end
end

-- returns the crafting station the player is in range of, or an empty string if there is none
function isPlayerInRangeofCraftingStation()

	local craftingStationName = ""
	local detectionRadius = 20
	local tag = "crafting_station"
	local craftingStation_x, craftingStation_y = 0
	
	local player_x, player_y = EntityGetTransform(getPlayerEntity())
	
	local craftingStationId = EntityGetInRadiusWithTag( player_x, player_y, detectionRadius, tag )
	
	if(craftingStationId ~= nil) then
		craftingStationName = EntityGetName(craftingStationId[1]) -- if there is more than one, only the first is obtained
		craftingStation_x, craftingStation_y = EntityGetTransform(craftingStationId[1])
	end
	
	return craftingStationName, craftingStation_x, craftingStation_y
end

function getWandNameAndTagFromWandID(gun_id)
	local name = ""
	local tag = ""
	if(EntityHasTag(gun_id, "custom_wand")) then
		name = EntityGetName(gun_id)
		tag = "custom_wand"
	elseif(EntityHasTag(gun_id, "boss_reward_wand")) then
		name = EntityGetName(gun_id)
		tag = "boss_reward_wand"
	else
		name = "[unknown]"
		tag = "[unknown]"
	end
		
	return name, tag
end

-- obtains name and custom wand tag of currently equipped wand
function getCurrentlyEquippedWandNameAndTag()
	local name = ""
	local tag = ""
	local gun_entity_id = -1
	for i, component in ipairs(EntityGetComponent(getPlayerEntity(), "Inventory2Component") or {}) do
		gun_entity_id = ComponentGetValue2(component, "mActiveItem")
	end
	if(EntityHasTag(gun_entity_id, "custom_wand")) then
		name = EntityGetName(gun_entity_id)
		tag = "custom_wand"
	elseif(EntityHasTag(gun_entity_id, "boss_reward_wand")) then
		name = EntityGetName(gun_entity_id)
		tag = "boss_reward_wand"
	else
		name = "[unknown]"
		tag = "[unknown]"
	end
		
	return name, tag
end

-- checks whether or not currently equipped wand is a wand from The Armory
function isCurrentWandTheArmory()
	local name, tag = getCurrentlyEquippedWandNameAndTag()
	
	if(tag ~= "[unknown]") then
		return true
	else
		return false
	end
end

-- gets id of currently equipped item
function getCurrentlyEquippedWandId()
	local gun_entity_id = -1
	for i, component in ipairs(EntityGetComponent(getPlayerEntity(), "Inventory2Component") or {}) do
		gun_entity_id = ComponentGetValue2(component, "mActiveItem")
	end
	return gun_entity_id
end

-- returns true if gun_id is the id of a wand that is currently equipped, false otherwise
function isWandEquipped(gun_id)
	local flag = false
	for i, component in ipairs(EntityGetComponent(getPlayerEntity(), "Inventory2Component") or {}) do
		local equipped_wand_id = ComponentGetValue2(component, "mActiveItem")
		if(equipped_wand_id == gun_id) then
			flag = true
		end
	end
	return flag
end

-- drops currently equipped item
function dropCurrentlyEquippedItem()
	local currentEquippedItem_id = getCurrentlyEquippedWandId()
	if(EntityHasTag( currentEquippedItem_id, "item" )) then
		local inventory2component = EntityGetFirstComponent(getPlayerEntity(), "Inventory2Component")
		local item_id = nil
		if inventory2component then
			item_id = ComponentGetValue(inventory2component, "mActiveItem")
			ComponentSetValue(inventory2component, "mActiveItem", "0")
		end
		EntityRemoveFromParent(item_id)
		EntitySetComponentsWithTagEnabled(item_id, "enabled_in_world", true)
		EntitySetComponentsWithTagEnabled(item_id, "enabled_in_hand", true)
		EntitySetComponentsWithTagEnabled(item_id, "enabled_in_inventory", true)
	end
end

-- simple utility function to return the player entity
function getPlayerEntity()
	local players = EntityGetWithTag("player_unit")
	if #players == 0 then return end

	return players[1]
end


-- this function returns the id of the shooter of a projectile with id equals to projectile_id
function getShooterId(projectile_id)
	local shooter_id
	local comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )
	if( comp_ids ~= nil ) then
		shooter_id = ComponentGetValue( comp_ids[1], "mWhoShot")
	end
	return shooter_id
end
-- this function returns the species/herd of the shooter of a projectile with id equals to projectile_id
function getShooterSpecies(projectile_id)
	local shooter_species
	local comp_ids = EntityGetComponent( projectile_id, "ProjectileComponent" )
	if( comp_ids ~= nil ) then
		shooter_species = ComponentGetValue( comp_ids[1], "mShooterHerdId")
	end
	return shooter_species
end

-- function to return the species of an enemy based on its entity
-- expects, obviously, the id of an enemy entity
function getSpeciesFromEnemyEntity(enemy_entity_id)
	local genomeComponent = EntityGetFirstComponent( enemy_entity_id, "GenomeDataComponent" )
	return ComponentGetValue( genomeComponent, "herd_id" )
end

function getSpeciesFromPlayer()
	local genomeComponent = EntityGetFirstComponent( getPlayerEntity(), "GenomeDataComponent" )
	return ComponentGetValue( genomeComponent, "herd_id" )
end

-- function to return the blood type of an enemy based on its entity
-- expects, obviously, the id of an enemy entity
function getBloodMaterialFromEnemyEntity(enemy_entity_id)
	local damageComponent = EntityGetFirstComponent(enemy_entity_id, "DamageModelComponent")
	return ComponentGetValue(damageComponent, "blood_spray_material")
end

-- function to verify if an entity is dropped on the ground or is equipped by the player
-- returns true if dropped, false otherwise
function isObjectDropped(entity_id)
	local parent = EntityGetParent(entity_id)
	if(parent == 0) then
		return true -- wand is dropped on the ground
	else
		return false -- wand is still equipped
	end
end

-- function to return a property's string value from VariableStorageComponent added to player entity, refer to init.lua to see where and how it initializes
function getStringValuePlayerStorageComp(varName)

	local components = EntityGetComponent( getPlayerEntity(), "VariableStorageComponent" )
	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == varName) then
				return ComponentGetValue( comp_id, "value_string" )
			end
		end
	end
end

-- function to update a property's string value from VariableStorageComponent added to player entity, refer to init.lua to see where and how it initializes
function setStringValuePlayerStorageComp(varName, newValue)

	local components = EntityGetComponent( getPlayerEntity(), "VariableStorageComponent" )
	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == varName) then
				ComponentSetValue( comp_id, "value_string", newValue )
			end
		end
	end
end

-- function to return a property's numeric value from VariableStorageComponent added to player entity, refer to init.lua to see where and how it initializes
function getNumericValuePlayerStorageComp(varName)

	local components = EntityGetComponent( getPlayerEntity(), "VariableStorageComponent" )
	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == varName) then
				return ComponentGetValueInt( comp_id, "value_int" )
			end
		end
	end
end

-- function to update a property's numeric value from VariableStorageComponent added to player entity, refer to init.lua to see where and how it initializes
function setNumericValuePlayerStorageComp(varName, newValue)

	local components = EntityGetComponent( getPlayerEntity(), "VariableStorageComponent" )
	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == varName) then
				ComponentSetValue( comp_id, "value_int", newValue )
			end
		end
	end
end

-- function to return a property's numeric value from a VariableStorageComponent added to an entity
-- varName is the name of variable being accessed
-- entity_id is the numeric id of the entity whose component will be accessed
-- var_type can be either value_int or value_string
function getValueStorageComponent(varName, entity_id, var_type)

	local components = EntityGetComponent( entity_id, "VariableStorageComponent" )
	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == varName) then
				if(var_type == "value_int") then
					return ComponentGetValueInt( comp_id, var_type )
				elseif(var_type == "value_string") then
					return ComponentGetValue( comp_id, var_type )
				elseif(var_type == "value_float") then
					return ComponentGetValueFloat( comp_id, var_type )	
				end
			end
		end
	end
end

-- function to update a property's numeric value from a VariableStorageComponent added to an entity
-- varName is the name of variable being set
-- entity_id is the numeric id of the entity whose component will be set
-- var_type can be either value_int or value_string
function setValueStorageComponent(varName, newValue, entity_id, var_type)

	local components = EntityGetComponent( entity_id, "VariableStorageComponent" )
	
	if ( components ~= nil ) then
		for key,comp_id in pairs(components) do 
			local var_name = ComponentGetValue( comp_id, "name" )
			if( var_name == varName) then
				ComponentSetValue( comp_id, var_type, newValue )
			end
		end
	end
end