dofile_once("data/scripts/game_helpers.lua")
dofile_once("data/scripts/lib/utilities.lua")
dofile_once("data/scripts/thearmory_generic_functions.lua")


-- checks for crafting materials before every crafting attempt
-- returns true if the player has at least one ingot and supplied crafting parts for the type specified
-- ingot_name is the internal ingot name e.g. ingot_iron, ingot_ruby or ingot_purple_fluorite
-- crafting_parts_name is the internal name of the crafting parts type e.g. crafting_parts_weapon, crafting_parts_trinket or crafting_parts_ammo
-- crafting_parts_amount is the amount of the crafting parts to be checked
function checkCraftingMaterials(ingot_name, crafting_parts_name, crafting_parts_amount)
	local flag = false
	
	if(getPlayerCraftingParts(ingot_name) >= 1 and getPlayerCraftingParts(crafting_parts_name) >= crafting_parts_amount) then
		flag = true
	end
	
	return flag
end

-- removes crafting materials from player, then spawns specified item at the feet of the player
-- item_path is the full path to the entity related to the item to ge given to the player
-- ingot_name is the internal ingot name e.g. ingot_iron, ingot_ruby or ingot_purple_fluorite
-- crafting_parts_name is the internal name of the crafting parts type e.g. crafting_parts_weapon, crafting_parts_trinket or crafting_parts_ammo
-- crafting_parts_amount is the amount of the crafting parts to be checke
function spawnItemandDeductMaterials(item_path, ingot_name, crafting_parts_name, crafting_parts_amount)
	
	addPlayerCraftingParts(-1, ingot_name)
	addPlayerCraftingParts(crafting_parts_amount * -1, crafting_parts_name)
	
	local x, y = EntityGetTransform(getPlayerEntity())
	
	EntityLoad(item_path, x, y)
end


-- function to add to player ammo
-- amount is how much ammo is being added, may be positive or negative
-- ammo_type is the type of the ammo being added (see get ammo functions below for reference)
-- ammo_type_name is the display name of the ammo type being added
function addPlayerAmmo(amount, ammo_type, ammo_type_name)
	if(ModSettingGet("TheArmory.infinite_ammo") == "enabled") then -- this setting comes from settings.lua
		return
	end
	local currentAmmo = getNumericValuePlayerStorageComp(ammo_type)
	currentAmmo = currentAmmo + amount
	--GamePrint("You now have ".. currentAmmo .." ammo")
	setNumericValuePlayerStorageComp(ammo_type, currentAmmo)
	if(currentAmmo <= 0) then
		returnMessage = "You have run out of " .. ammo_type_name.. ", collect more to shoot"
		GamePrintImportant( returnMessage, "" )
		GamePrint(returnMessage)
	end
end

-- function to check if the player has ammo equivalent to amount, of given ammo_type, with the ammo_type_name label
-- returns true if the player has that amount, or false otherwise
function checkPlayerAmmo(amount, ammo_type, ammo_type_name) 
	if(amount <= getNumericValuePlayerStorageComp(ammo_type)) then
		return true
	else
		GamePrint("This spell requires " .. amount .. " " .. ammo_type_name)
		return false
	end
end

-- function to check whether or not given ammo_type is currently depleted
function checkAmmoDepleted(ammo_type)
	if(getNumericValuePlayerStorageComp(ammo_type) == 0) then
		return true
	else
		return false
	end
end

-- function to return current ammo of all sorts, ammo types can be:
-- pistol_ammo
-- smg_ammo
-- shotgun_ammo
-- ar_rifle_ammo
-- rifle_ammo
-- revolver_ammo
-- energy_cells
-- grenades_ammo
-- nails_ammo
-- fluid_ammo
-- mininukes_ammo
-- elementalbullets_ammo
-- knives_ammo
-- metalsticks_ammo
-- rockets_ammo
-- shovels_ammo
-- syringes_ammo
-- trash_ammo
-- arrows_ammo
-- crossbowbolts_ammo
-- spears_ammo
-- fuel_ammo
-- throwable_weapons_ammo
-- throwable_explosives_ammo
function getPlayerAmmo(ammo_type)
	return getNumericValuePlayerStorageComp(ammo_type)
end

-- function to add enemy guts to player inventory
-- amount is how many are being added, may be positive or negative
function addPlayerEnemyGuts(amount)
	local enemyGuts = getNumericValuePlayerStorageComp("enemyguts")
	enemyGuts = enemyGuts + amount
	--GamePrint("You now have ".. enemyGuts .." enemy guts")
	setNumericValuePlayerStorageComp("enemyguts", enemyGuts)
end

-- function to obtain current amount of enemy guts in possession of the player
function getPlayerEnemyGuts()
	return getNumericValuePlayerStorageComp("enemyguts")
end

-- function to add slot machine chips to player inventory
-- amount is how many chips are being added, may be positive or negative
function addPlayerSlotMachineChips(amount)
	local currentChips = getNumericValuePlayerStorageComp("slotmachinechips")
	currentChips = currentChips + amount
	--GamePrint("You now have ".. currentChips .." slot machine chips")
	setNumericValuePlayerStorageComp("slotmachinechips", currentChips)
end

-- function to obtain current amount of slot machine chips in possession of the player
function getPlayerSlotMachineChips()
	return getNumericValuePlayerStorageComp("slotmachinechips")
end

-- function to add crafting parts to player inventory
-- amount is how many crafting parts are being added
-- crafting part items include crafting parts themselves, ingots and ore
function addPlayerCraftingParts(amount, crafting_part_type)
	local currentCraftingParts = getNumericValuePlayerStorageComp(crafting_part_type)
	currentCraftingParts = currentCraftingParts + amount
	setNumericValuePlayerStorageComp(crafting_part_type, currentCraftingParts)
end

-- returns current amount of crafting parts in possession of the player, according to crafting_part_type
function getPlayerCraftingParts(crafting_part_type)
	return getNumericValuePlayerStorageComp(crafting_part_type)
end

-- function to add safe keys to player inventory
-- amount is how many safe keys are being added, may be positive or negative
function addPlayerKeys(amount, key_type)
	local currentKeys = getNumericValuePlayerStorageComp(key_type)
	currentKeys = currentKeys + amount
	setNumericValuePlayerStorageComp(key_type, currentKeys)
end

-- function to obtain current amount of safe keys in possession of the player
function getPlayerSafeKeys()
	return getNumericValuePlayerStorageComp("safekeys")
end

-- function to obtain current amount of chest keys in possession of the player
function getPlayerChestKeys()
	return getNumericValuePlayerStorageComp("chestkeys")
end

-- function to obtain current amount of miniboss keys in possession of the player
function getPlayerMinibossKeys()
	return getNumericValuePlayerStorageComp("minibosskeys")
end

-- function to return player health
-- health values are treated internally as floats, so if the player has 5 health, the internal value will be 0.008
function getPlayerHealth()
	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	local health = 0
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			health = tonumber( ComponentGetValue( v, "hp" ) )
			--GamePrint(health)
			break
		end
	end
	return health
end

-- obtains the percentage of current player health compared to total health
function getPlayerCurrentHealthPercentage()
	local player_health = getPlayerHealth()
	local player_max_health = getPlayerMaxHealth()
	local current_health_percentage = math.floor(player_health * 100 / player_max_health)
	--GamePrint("current health percentage is " .. current_health_percentage)
	return current_health_percentage
end

-- this function updates player health based on provided percentage
-- always calculated based on the current health, not max health
-- percentage must be an integer from 0 to 100
function addPlayerHealthByPercentage(percentage)
	local player_health = getPlayerHealth()
	local bonus_health = player_health * percentage / 100
	
	GamePrint("bonus health: " .. bonus_health)
	
	setPlayerHealth(player_health + bonus_health)
end

-- function to return player max health
function getPlayerMaxHealth()
	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	local maxHealth = 0
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			maxHealth = tonumber( ComponentGetValue( v, "max_hp" ) )
			
			break
		end
	end
	return maxHealth
end

-- function to add to player health
-- amount is how much is player health to be added to, may be positive or negative
-- amount must be provided as an integer
function addPlayerHealth(amount, overheal_flag)

	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
		
			currentHealth = tonumber( ComponentGetValue( v, "hp" ) )
			currentMaxHealth = tonumber( ComponentGetValue( v, "max_hp") )
			
			local trueAmount = amount * 0.04 -- How health scaling works when shown by UI
			local updatedHealth = currentHealth + trueAmount
			local updatedMaxHealth = 0
			
			local updatedHealth = currentHealth + trueAmount
			if(updatedHealth > currentMaxHealth and overheal_flag) then -- overhealing, if new health goes beyond max health, max health is updated
				updatedMaxHealth = updatedHealth
				ComponentSetValue( v, "max_hp", updatedMaxHealth)
				ComponentSetValue( v, "hp", updatedHealth)
			else
				if(updatedHealth <= 0) then
					updatedHealth = 0.04
					--GamePrint("player health below zero, updated health is " .. updatedHealth)
				end
				ComponentSetValue( v, "hp", updatedHealth)
				
				-- no longer kills the player, but /\ sets health to 1 instead when updatedHealth is zero or below
				--[[
				if(updatedHealth <= 0) then
					EntityKill(getPlayerEntity())
				end
				]]
			end
			
			break
		end
	end

end

-- function to directly set a value to player health
-- newHealth is the new health the player is going to have
-- newHealth must be provided as a float value
function setPlayerHealth(newHealth)
	local damagemodels = EntityGetComponent( getPlayerEntity(), "DamageModelComponent" )
	
	if( damagemodels ~= nil ) then
		for i,v in ipairs(damagemodels) do
			ComponentSetValue( v, "hp", newHealth)
		end
	end
end

-- function to add or subtract gold from player
-- amount can be either negative or positive
function addPlayerGold(amount)

	local player = getPlayerEntity()

	local wallet = EntityGetFirstComponent(player, "WalletComponent")
	local money = ComponentGetValueInt(wallet, "money")
	local player_money = money + amount
	-- extra money perk scale
	-- amount = amount * (2 ^ GameGetGameEffectCount(player, "EXTRA_MONEY"))
	
	--ComponentSetValueInt(wallet, "money", money + amount)
	edit_component(
	player,
	"WalletComponent",
	function(comp,vars)
		vars.money = player_money
	end
	)
end

-- function to get player gold
function getPlayerGold()
	
	local player = getPlayerEntity()
	
	local wallet = EntityGetFirstComponent(player, "WalletComponent")
	local money = ComponentGetValueInt(wallet, "money")

	return money
end

-- function to add items to player inventories
-- entity_path is the complete entity path to the item being added
-- inventory_type can be either "inventory_quick" or "inventory_full"
function addItemtoPlayerInventory(entity_path, inventory_type)

	local children = EntityGetAllChildren(getPlayerEntity())
	local inventory = nil
    if children == nil then return end
    for i,child in ipairs(children) do
		if(EntityGetName(child) == inventory_type) then
			inventory = child
		end
	end
	
	local newItem = EntityLoad(entity_path)
	EntityAddChild(inventory, newItem)
end




-- function to show the player an important message
-- it is shown mid-screen and as a console entry
function showImportantMessage(messageTitle, message)
	GamePrintImportant(messageTitle, message)
	GamePrint(message)
end


-- Unused functions, but kept for future reference
--------------------------------------------------
-- function to calculate a chance based on a percent value
-- chance must be provided in float format, from 0.0 to 1.0
-- it returns true or false depending on what random number was generated
function percentChance(chance)

	local r = math.random()
	
	if r < chance then
		return true
	else
		return false
	end
end

-- function to remove ghost items from inventory (this can happen when a vending machine is interacted with)
-- entity tag is the targeted tag of the item to be removed
function killGhostItems(target_entity_tag)

	player_entity = getPlayerEntity()

	local inventory = get_entity_child(player_entity, "inventory_quick")

	local inventoryItems = EntityGetAllChildren(inventory)
	for i,item in ipairs(inventoryItems) do
		if EntityHasTag(item, target_entity_tag) then --items tagged with target_entity_tag are removed
			GamePrint("vending machine removed from inventory")
			GameKillInventoryItem(player_entity, item)
		end
	end
end

-- function to load a pocket of a given material
-- materialName is the string name of the material (names can be found at data/materials.xml)
-- size can be small, medium, big or huge
-- x and y are the coordinates where the entity must be loaded
-- Important: an entity file is required when calling this function
function loadMaterialPocket(materialName, size, x, y)
	GamePrint("material " .. materialName .. " loaded")
	EntityLoad("mods/TheArmory/data/entities/" .. materialName .. "_pocket_" .. size .. ".xml", x, y)
end

-- function to add ore to player inventory
-- amount is how many are being added, may be positive or negative
function addPlayerOre(amount, ore_var_name)
	local oreCount = getNumericValuePlayerStorageComp(ore_var_name)
	oreCount = oreCount + amount
	setNumericValuePlayerStorageComp(ore_var_name, oreCount)
end

-- function to obtain current amount of ore in possession of the player
function getPlayerOre(ore_var_name)
	return getNumericValuePlayerStorageComp(ore_var_name)
end

-- function to add ore to player inventory
-- amount is how many are being added, may be positive or negative
function addPlayerIngot(amount, ingot_var_name)
	local ingotCount = getNumericValuePlayerStorageComp(ingot_var_name)
	ingotCount = ingotCount + amount
	setNumericValuePlayerStorageComp(ingot_var_name, ingotCount)
end

-- function to obtain current amount of ingots in possession of the player
function getPlayerIngot(ingot_var_name)
	return getNumericValuePlayerStorageComp(ingot_var_name)
end
