Param (
	[string]$material = $(Read-Host "Slime material"),
	[string]$explosion_name = $(Read-Host "Explosion name"),
	[string]$particle_material_name = $(Read-Host "Projectile Particle Material name")
)

# Slime Sprite files, variables: \[slime_sprite_file\]
$slime_sprite_xml_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalslime_sprite_dummy_file.xml"
$slime_sprite_png_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalslime_sprite_dummy_file.png"

# Projectile sprite files, variables: \[projectile_sprite_file\]
$projectile_sprite_xml_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalorb_sprite_dummy_file.xml"
$projectile_sprite_png_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalorb_sprite_dummy_file.png"

# Slime Entity file, variables: \[slime_name\], \[slime_blood_material\], \[projectile_name\], \[explosion_name\], \[material_pocket_name\]
$slime_entity_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalslime_dummy_file.xml"
$slime_entity_dummy_friendly = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalslime_dummy_file_friendly.xml"

# Projectile Entity file, variables: \[explosion_name\], \[material_lines_name\], \[projectile_name\], \[particle_material_name\]
$projectile_entity_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elemental_slime\elementalorb_dummy_file.xml"

# variable definitions
$projectile_sprite_file = "elementalorb_" + $material.Replace("_", "")
$projectile_name = $projectile_sprite_file
$slime_sprite_file = "elementalslime_" + $material.Replace("_", "")
$slime_name = $slime_sprite_file
$slime_blood_material = $material
$material_pocket_name = $material + "_pocket_medium"
$material_lines_name = $material + "_lines"

# new file names
$projectile_xml_file_name = $projectile_sprite_file + ".xml"
$slime_xml_file_name = $slime_sprite_file + ".xml"
$slime_xml_file_name_friendly = $slime_sprite_file + "_friendly.xml"
$projectile_png_file_name = $projectile_sprite_file + ".png"
$slime_png_file_name = $slime_sprite_file + ".png"

# Default paths for new files
$path_enemies_entities = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\entities\animals\" + $slime_xml_file_name
$path_enemies_entities_friendly = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\entities\animals\" + $slime_xml_file_name_friendly
$path_projectiles_entities = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\entities\projectiles\enemy_projectiles\" + $projectile_xml_file_name

$path_enemies_gfx_png = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\enemies_gfx\" + $slime_png_file_name
$path_projectiles_gfx_png = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\projectiles_gfx\" + $projectile_png_file_name

$path_enemies_gfx_xml = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\enemies_gfx\" + $slime_xml_file_name
$path_projectiles_gfx_xml = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\projectiles_gfx\" + $projectile_xml_file_name

# Copying files from dummy to new paths
Copy-Item $slime_sprite_xml_dummy -Destination $path_enemies_gfx_xml
Copy-Item $slime_sprite_png_dummy -Destination $path_enemies_gfx_png

Copy-Item $projectile_sprite_xml_dummy -Destination $path_projectiles_gfx_xml
Copy-Item $projectile_sprite_png_dummy -Destination $path_projectiles_gfx_png

Copy-Item $slime_entity_dummy -Destination $path_enemies_entities
Copy-Item $slime_entity_dummy_friendly -Destination $path_enemies_entities_friendly
Copy-Item $projectile_entity_dummy -Destination $path_projectiles_entities

# Replacing variables with values
((Get-Content -path $path_enemies_gfx_xml -Raw) -replace '\[slime_sprite_file\]', $slime_sprite_file) | Set-Content -Path $path_enemies_gfx_xml
((Get-Content -path $path_projectiles_gfx_xml -Raw) -replace '\[projectile_sprite_file\]', $projectile_sprite_file) | Set-Content -Path $path_projectiles_gfx_xml

((Get-Content -path $path_enemies_entities -Raw) -replace '\[slime_name\]', $slime_name) | Set-Content -Path $path_enemies_entities
((Get-Content -path $path_enemies_entities -Raw) -replace '\[slime_blood_material\]', $slime_blood_material) | Set-Content -Path $path_enemies_entities
((Get-Content -path $path_enemies_entities -Raw) -replace '\[projectile_name\]', $projectile_name) | Set-Content -Path $path_enemies_entities
((Get-Content -path $path_enemies_entities -Raw) -replace '\[explosion_name\]', $explosion_name) | Set-Content -Path $path_enemies_entities
((Get-Content -path $path_enemies_entities -Raw) -replace '\[material_pocket_name\]', $material_pocket_name) | Set-Content -Path $path_enemies_entities

((Get-Content -path $path_enemies_entities_friendly -Raw) -replace '\[slime_name\]', $slime_name) | Set-Content -Path $path_enemies_entities_friendly
((Get-Content -path $path_enemies_entities_friendly -Raw) -replace '\[slime_blood_material\]', $slime_blood_material) | Set-Content -Path $path_enemies_entities_friendly
((Get-Content -path $path_enemies_entities_friendly -Raw) -replace '\[projectile_name\]', $projectile_name) | Set-Content -Path $path_enemies_entities_friendly
((Get-Content -path $path_enemies_entities_friendly -Raw) -replace '\[explosion_name\]', $explosion_name) | Set-Content -Path $path_enemies_entities_friendly
((Get-Content -path $path_enemies_entities_friendly -Raw) -replace '\[material_pocket_name\]', $material_pocket_name) | Set-Content -Path $path_enemies_entities_friendly

((Get-Content -path $path_projectiles_entities -Raw) -replace '\[explosion_name\]', $explosion_name) | Set-Content -Path $path_projectiles_entities
((Get-Content -path $path_projectiles_entities -Raw) -replace '\[material_lines_name\]', $material_lines_name) | Set-Content -Path $path_projectiles_entities
((Get-Content -path $path_projectiles_entities -Raw) -replace '\[projectile_name\]', $projectile_name) | Set-Content -Path $path_projectiles_entities
((Get-Content -path $path_projectiles_entities -Raw) -replace '\[particle_material_name\]', $particle_material_name) | Set-Content -Path $path_projectiles_entities