# this script is used to create all necessary files and list entries for an elemental variation of an existent weapon

Param (
	[string]$weapon = $(Read-Host "What weapon to create"),
	[string]$material = $(Read-Host "Projectile material starting with Capital e.g. Acid, Water, etc")
)

$actionsFilePath = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\files\actions.lua"
$weaponsFilePath = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\files\wands_list.lua"

if($weapon -eq "elementalrifle") { # creates elemental rifle variation

	# dummy files
	$weaponFile_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elementalrifle\elementalrifle_dummy.xml"
	$actionCardFile_dummy = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\dummy_files\elementalrifle\action_card_dummy.png"
	
	# fixed names
	$material_uiname = $material
	$material_filename = $material.ToLower().Replace(" ", "")
	$material_actionname_withspaces = $material.ToUpper()
	$material_actionname = $material.ToUpper().Replace(" ", "")
	
	# destination paths
	$weaponFile = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\entities\items\elementalrifle_" + $material_filename + ".xml"
	$actionCardFile = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\files\actions\elementalorb_" + $material_filename + ".png"

	# list entry strings
	$actionEntry = 'table.insert( actions,
{
	id          = "THEARMORYACTIONS_ELEMENTALORB_[material_actionname]",
	name 		= "ELEMENTAL ORB ([material_actionname_withspaces])",
	description = "An elemental orb that explodes spitting [material_filename]",
	sprite 		= "mods/TheArmory/files/actions/elementalorb_[material_filename].png",
	type 		= ACTION_TYPE_MATERIAL,
	spawn_level                       = "0", -- THEARMORYACTIONS_ELEMENTALORB_[material_actionname]
	spawn_probability                 = "0", -- THEARMORYACTIONS_ELEMENTALORB_[material_actionname]
	price = 600,
	mana = 20,
	action 		= function()
		
		if(getPlayerEntity() ~= nil) then
			if(checkPlayerAmmo(1, "energy_cells", "Energy Cells")) then
				
				applyTrinketsEffectsWeapons_Energy("elementalorb_[material_filename].xml")
				
				applyTrinketsEffectsWeapons_StackableBonusesRanged("elementalorb_[material_filename].xml")

				add_projectile("data/entities/projectiles/elementalorb_[material_filename].xml")
				
				addPlayerAmmo(applyTrinketsEffectsAmmo_Consumption(-1), "energy_cells", "Energy Cells")
			end
		end
		
	end,
} )

--#NEW_ENTRY#'

	$actionEntry = $actionEntry.Replace("[material_filename]", $material_filename)
	$actionEntry = $actionEntry.Replace("[material_actionname]", $material_actionname)
	$actionEntry = $actionEntry.Replace("[material_actionname_withspaces]", $material_actionname_withspaces)
	
	$weaponEntry = '{
		tier = "exotic",
		crafting_tier_item = "ingot_ruby",
		crafting_parts_cost = 50,
		description = "Comes with ELEMENTAL ORB ([material_actionname_withspaces]) Spell - An elemental orb that explodes spitting [material_filename]",
		file_name = "elementalrifle_[material_filename].xml",
		bullet_casing = "[none]",
		bullet_casing_amount = 0,
		empty_magazine = "data/entities/props/casings_and_magazines/empty_magazine_energy_cell.xml",
		empty_magazine_amount = 1,
		ui_name="Elemental Rifle ([material_uiname])",
		thumbnail = "data/items_gfx/elementalrifle.png",
		ammo_type = "energy_cells",
		trinket_types = "energy,elemental,"
	},
	
	--#NEW_ENTRY#'
	
	$weaponEntry = $weaponEntry.Replace("[material_filename]", $material_filename)
	$weaponEntry = $weaponEntry.Replace("[material_actionname]", $material_actionname)
	$weaponEntry = $weaponEntry.Replace("[material_uiname]", $material_uiname)
	$weaponEntry = $weaponEntry.Replace("[material_actionname_withspaces]", $material_actionname_withspaces)
	
	# Copying dummy files to new paths
	Copy-Item $weaponFile_dummy -Destination $weaponFile
	Copy-Item $actionCardFile_dummy -Destination $actionCardFile
	
	# Replacing variables with values
	((Get-Content -path $weaponFile -Raw) -replace '\[material_name\]', $material_filename) | Set-Content -Path $weaponFile
	((Get-Content -path $weaponFile -Raw) -replace '\[material_uiname\]', $material_uiname) | Set-Content -Path $weaponFile
	((Get-Content -path $weaponFile -Raw) -replace '\[material_actionname\]', $material_actionname) | Set-Content -Path $weaponFile
	
	# Adding list entries
	((Get-Content -path $actionsFilePath -Raw) -replace '--#NEW_ENTRY#', $actionEntry) | Set-Content -Path $actionsFilePath
	((Get-Content -path $weaponsFilePath -Raw) -replace '--#NEW_ENTRY#', $weaponEntry) | Set-Content -Path $weaponsFilePath
	
	# Write-Host $actionEntry -f "green"
}