$actionsFile = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\files\actions.lua"
$wandsListFile = "C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\files\wands_list.lua"
$wandFiles = Get-ChildItem 'C:\Program Files (x86)\Steam\steamapps\common\Noita\mods\TheArmory\data\entities\items' | Where-Object { !$_.PSIsContainer }

# Counter for files found within $wandFiles
$fileCount = 0

# line break with carriage return characters
$lineBreak = "`r`n"

$ammoType = ''
$trinketTypes = ''
$wandName = ''

foreach($wandFile in $wandFiles) {

	$wandName = $wandFile.BaseName
	
	<#
	# remove this
	if($wandName -ne 'aa12') {
		break
	}
	# remove this
	#>
	
	# Obtains individual action from wand file
	$query = Select-String -Path $wandFile.FullName -Pattern 'add_these_child_actions=(.*)' | Select-Object -ExpandProperty Line
	$queryParts = $query.Split('=').Replace('"', '') # separates prop name from value
	$actionsList = $queryParts[1] # obtains list of actions bound to the wand
	$actionName = ""
	if($actionsList.Contains(',')) 
	{
		$actionsListParts = $actionsList.Split(',') # separates repeated actions so that just one can be obtained
		$actionName = $actionsListParts[1]
	} else {
		$actionName = $actionsList
	}
	
	# Obtains ammo and trinket types from actions.lua corresponding to $actionName
	
	$actionNamePattern = '= "' + $actionName + '",'
	
	$ammoType = "[none]"
	
	$lineNumber = 1

	$blockEndMarkFound = $false
	$block = ''

	$trinketTypesStrBuilder = [System.Text.StringBuilder]::new()

	do {
		Select-String -Path $actionsFile -Pattern $actionNamePattern -Context 0,$lineNumber | ForEach-Object {
			$blockEndQuery = $_.Context.PostContext | Select-String -Pattern '} \)'
			if($blockEndQuery -ne $null) {
				$blockEndMarkFound = $true
				$block = $_.Context.PostContext	
			} else {
				$lineNumber++
			}
		}
	} while(!$blockEndMarkFound)

	$ammoTypeQuery = $block | Select-String -Pattern 'checkPlayerAmmo\(.*, "(.*)", ".*"\)'
	if($ammoTypeQuery -ne $null) {
		$ammoType = $ammoTypeQuery.Matches.Groups[1].Value
	}	
	
	$trinket_singleShotQuery = $block | Select-String -Pattern 'applyTrinketsEffectsWeapons_SingleShotRegular'
	$trinket_scatterShotQuery = $block | Select-String -Pattern 'applyTrinketsEffectsWeapons_ScatterShotRegular'
	$trinket_energyQuery = $block | Select-String -Pattern 'applyTrinketsEffectsWeapons_Energy'
	$trinket_elementalQuery = $block | Select-String -Pattern 'applyTrinketsEffectsWeapons_Elemental'
	$trinket_meleeQuery = $block | Select-String -Pattern 'applyTrinketsEffectsWeapons_Melee'
	$trinket_explosiveQuery = $block | Select-String -Pattern 'applyTrinketsEffectsWeapons_Explosive'

	if($trinket_singleShotQuery -ne $null) {
		[void]$trinketTypesStrBuilder.Append('single_shot_regular,')
	}
	if($trinket_scatterShotQuery -ne $null) {
		[void]$trinketTypesStrBuilder.Append('scatter_shot_regular,')
	}
	if($trinket_energyQuery -ne $null) {
		[void]$trinketTypesStrBuilder.Append('energy,')
	}
	if($trinket_elementalQuery -ne $null) {
		[void]$trinketTypesStrBuilder.Append('elemental,')
	}
	if($trinket_meleeQuery -ne $null) {
		[void]$trinketTypesStrBuilder.Append('melee,')
	}
	if($trinket_explosiveQuery -ne $null) {
		[void]$trinketTypesStrBuilder.Append('explosive,')
	}
	
	if($trinketTypesStrBuilder.ToString() -eq "" -or $trinketTypesStrBuilder.ToString() -eq $null) {
		$trinketTypes = "[none]"
	} else {
		$trinketTypes = $trinketTypesStrBuilder.ToString()
	}
	
	
	<#
	# searches for the action id in actions.lua, then obtains the first 14 lines below it, this is done in order for the line mentioning ammo type to be captured for the next step
	Select-String -Path $actionsFile -Pattern $actionNamePattern -Context 0,14 | ForEach-Object {
		# if the text block contains a call to checkPlayerAmmo, captures it and separates ammo type identifies from the rest
		if($_.Context.PostContext.Trim() -match 'checkPlayerAmmo\(.*, "(.*)", ".*"\)') {
			$line = $_.Context.PostContext.Trim() | Select-String -Pattern 'checkPlayerAmmo\(.*, "(.*)", ".*"\)'
			$ammoType = $line.Matches.Groups[1].Value
		}
	}
	#>
	
	Write-Host ($wandFile.Name) -f "cyan" -NoNewLine
	Write-Host  " action " -NoNewLine
	Write-Host ($actionName) -f "yellow" -NoNewLine
	Write-Host  " ammo type " -NoNewLine
	Write-Host ($ammoType) -f "green" -NoNewLine
	Write-Host " trinket types " -NoNewLine
	Write-Host ($trinketTypes) -f "magenta"
	Write-Host  " ---------- "
	
	# replaces for wand file
	# ----------------------
	# replace pattern for 'tags="custom_wand"'
	$replacePattern1 = 'name="' + $wandName + '" tags="custom_wand"'
	# replace pattern for '</Entity>'
	# $replacePattern2 = '	<LuaComponent' + $lineBreak + '		_enabled="1"' + $lineBreak + '		script_source_file="data/scripts/wands/lock.lua"' + $lineBreak + '		execute_on_added="1"' + $lineBreak + '		execute_times="1"' + $lineBreak + '		remove_after_executed="1"' + $lineBreak + '		execute_every_n_frame="-1">' + $lineBreak + '	</LuaComponent>' + $lineBreak + $lineBreak + '	<VariableStorageComponent' + $lineBreak + '		name="ammo_type"' + $lineBreak + '		value_string="' + $ammoType + '">' + $lineBreak + '	</VariableStorageComponent>' + $lineBreak + $lineBreak + '	<VariableStorageComponent' + $lineBreak + '		name="trinket_types"' + $lineBreak + '		value_string="' + $trinketTypes + '">' + $lineBreak + '	</VariableStorageComponent>' + $lineBreak + '</Entity>'
	$replacePattern2 = '	<LuaComponent' + $lineBreak + '		_enabled="1"' + $lineBreak + '		script_source_file="data/scripts/wands/lock.lua"' + $lineBreak + '		execute_on_added="1"' + $lineBreak + '		execute_times="1"' + $lineBreak + '		remove_after_executed="1"' + $lineBreak + '		execute_every_n_frame="-1">' + $lineBreak + '	</LuaComponent>' + $lineBreak + '</Entity>'
	
	((Get-Content -path $wandFile.FullName -Raw) -replace 'tags="custom_wand"', $replacePattern1) | Set-Content -Path $wandFile.FullName
	((Get-Content -path $wandFile.FullName -Raw) -replace '</Entity>', $replacePattern2) | Set-Content -Path $wandFile.FullName
	
	# replace for wand list file
	# --------------------------
	$listEntry = 'file_name = "' + $wandFile.Name + '"'
	$replacePattern3 = $listEntry + ',' + $lineBreak + '		ammo_type = "' + $ammoType + '",' + $lineBreak + '		trinket_types = "' + $trinketTypes + '"'
	
	((Get-Content -path $wandsListFile -Raw) -replace $listEntry, $replacePattern3) | Set-Content -Path $wandsListFile
	
	$fileCount++
}

Write-Host $fileCount -f c